#!/usr/bin/python
"""
Trade Game
by Kris Schnee

"""

##___________________________________
##          ~Header~
##___________________________________

__author__ = "Kris Schnee"
__version__ = "2008.6.1"
__license__ = "GPL"


##___________________________________
##      ~Imported Modules~
##___________________________________

## Standard
import random
random.seed("Porcupine Sea")

## Third-party
import pygame
from pygame.locals import *

## Mine
import ringtale
import driftwood
import art
##import conch
##from conch import MUSIC_END_EVENT ## Trigger an event when music ends.
import trader

##___________________________________
##            ~Constants~
##___________________________________

SCREEN_SIZE = (800,600)
art.fonts["default"] = art.fonts["EagerNaturalist"]
art.interface["default"] = art.interface["Bamboo"]
screen = pygame.display.set_mode(SCREEN_SIZE)

PLAYER_INVENTORY = {"meat":20,"flour":30,"bread":10,"iron":2,"brick":10,"grape":3,"emerald":5}

GOODS = ['pearl_necklace', 'grape', 'apple', 'flour', 'fish', 'emerald', 'meat', 'iron', 'brick', 'bread']

PORTS = [("Wheaton",(70,30),["flour","bread"],"Grayni"),
         ("Fruitburg",(200,400),["apple","grape"],"Stiliyf"),
         ("Emerald",(400,200),["emerald"],"Chao"),
         ("Bricksport",(580,280),["brick"],"Katan"),
         ]

art.portraits["yes"].set_alpha(150)
art.portraits["no"].set_alpha(150)

INTRO_TEXT = """Welcome! This is a demo of a little game about trading. Your mission is to travel to various ports, trying to increase your wealth by trading with the locals.
Go to the ports and offer goods from your inventory, then hit Offer. To indicate that you do/don't want something, click on the trader's offer or inventory.
Each merchant has their own idea of the value of things. Can you increase your wealth as measured by my arbitrary estimate below (the basis for merchants' estimates)? You'll see a final estimate when you quit.\n"""

##___________________________________
##            ~Classes~
##___________________________________

class GameWorld:
    def __init__(self):
        self.ports = {}
        for port in PORTS:
            t = trader.Trader(name=port[3],
                              tribe=port[0],
                              inventory={},
                              portrait=art.portraits["trader_"+port[3].lower()])
            ## Stock with low-priced goods produced here.
            for good in port[2]:
                t.values[good] *= .5
                t.inventory.Add(good,random.randint(20,60))
            ## Add some other goods randomly.
            for good in [good for good in GOODS if not good in port[2]]:
                if random.randint(0,99) < 50:
                    t.inventory.Add(good,random.randint(1,20))
                    t.values[good] *= 1.5

            p = {"name":port[0],
                 "trader":t,
                 }

            self.ports[port[0]] = p




class ItemButton( driftwood.Button ):
    """A widget that sends messages to its owner when clicked."""
    def __init__(self,**options):
        driftwood.Button.__init__(self,**options)
        self.surface.set_colorkey(self.background_color)
        self.wanted = options.get("wanted",0) ## -1, 0 or 1

    def Redraw(self):
        self.surface.fill(self.background_color)
        if self.visible:
            if self.background_image:
                self.surface.blit(self.background_image,(0,0))
            align_by = (50,10)
            self.surface.blit( self.rendered_text, align_by )

            if self.wanted == 1:
                self.surface.blit(art.portraits["yes"],(50,0))
            elif self.wanted == -1:
                self.surface.blit(art.portraits["no"],(50,0))

    def SetWanted(self,wanted=0):
        self.wanted = wanted
        self.Redraw()




class Game( ringtale.Framework ):
    def __init__(self):
        ringtale.Framework.__init__(self)
        driftwood.DEFAULT_OWNER = self ## I own widgets.

        self.player = trader.Trader(name="Kay",inventory=PLAYER_INVENTORY,tribe="Snow Plume")

        self.world = GameWorld()

        ## Start the Map screen, with a Wealth screen before and after.
        self.PushState("Wealth")
        self.PushState("Map")
        self.notes["intro"] = True
        self.PushState("Wealth",["intro"])
        print self.states


    def WealthSetup(self):
        text = ""
        if self.notes.get("intro"):
            text += INTRO_TEXT
        text += "Your estimated wealth: "+str(self.player.Appraise(self.player.inventory))+"\nHit any key to continue."
        self.AddWidget(driftwood.TextBox(name="tWealth",coords=(0,0,800,600),text=text))

    def WealthLogic(self):
        for event in pygame.event.get():
            if event.type in (KEYDOWN, MOUSEBUTTONDOWN):
                self.PopState()

    def WealthDraw(self):
        screen.fill((0,0,0))
        for i in self.interface:
            i.Draw()

    def TradeSetup(self):
        trader = self.notes["trader"]

        self.notes.setdefault("goods_multiple",1) ## Clicking adds/removes this many.
        self.ResetInterface()

        ## Trader's portrait and commentary.
        about_text = trader.name+"\n"+trader.tribe
        comments = " ".join(trader.comments)
        self.AddWidget(driftwood.Widget(name="portrait",coords=(400,0,150,150),background_image=trader.portrait,border=None))
        self.AddWidget(driftwood.TextBox(name="wAbout",coords=(550,0,250,150),text=about_text))
        self.AddWidget(driftwood.TextBox(name="wTalk",coords=(400,150,400,150),text=comments))

        ## Player
        about_text = self.player.name+"\n"+self.player.tribe
        self.AddWidget(driftwood.Widget(name="wPlayer",coords=(0,0,150,150),background_image=art.portraits["kl_trader"],border=None))
        self.AddWidget(driftwood.TextBox(name="wAboutPlayer",coords=(150,0,250,150),text=about_text))

        self.AddWidget(driftwood.Widget(name="wPlayerGoods",coords=(0,300,300,300)))
        self.AddWidget(driftwood.Widget(name="wOffers",coords=(300,300,200,300),border=None,background_image=art.posters["trademat"]))
        self.AddWidget(driftwood.Widget(name="wTraderGoods",coords=(500,300,300,300)))

        ## Other buttons.
        self.AddWidget(driftwood.Button(name="bOffer",coords=(0,150,150,50),text="Offer"))
        self.AddWidget(driftwood.Button(name="bAccept",coords=(0,200,150,50),text="Accept"))
        self.AddWidget(driftwood.Button(name="bMultiple",coords=(0,250,150,50),text="(X"+str(self.notes["goods_multiple"])+")"))

        xi, yi = 0, 310
        x, y = 0, 0
        for key in self.player.inventory:
            quantity = self.player.inventory[key]
            self.AddWidget(ItemButton(name="item_"+key,coords=(xi+x,yi+y,100,50),border=None,background_image=art.portraits.get(key,art.portraits["fish"]),text=str(quantity)))
            x += 100
            if x > 251:
                x = 0
                y += 50

        xi, yi = 300, 310
        x, y = 0, 0
        for key in self.player.offer:
            quantity = self.player.offer[key]
            self.AddWidget(ItemButton(name="offer_"+key,coords=(xi+x,yi+y,100,50),border=None,background_image=art.portraits.get(key,art.portraits["fish"]),text=str(quantity)))
            y += 50

        xi, yi = 400, 310
        x, y = 0, 0
        for key in trader.offer:
            quantity = trader.offer[key]
            self.AddWidget(ItemButton(name="trader_offer_"+key,coords=(xi+x,yi+y,100,50),border=None,background_image=art.portraits.get(key,art.portraits["fish"]),text=str(quantity)))
            y += 50

        xi, yi = 500, 310
        x, y = 0, 0
        for key in trader.inventory:
            quantity = trader.inventory[key]
            wanted = 0
            if key in trader.customer["wants"]:
                wanted = 1
            elif key in trader.customer["does_not_want"]:
                wanted = -1
            self.AddWidget(ItemButton(name="trader_item_"+key,coords=(xi+x,yi+y,100,50),border=None,background_image=art.portraits.get(key,art.portraits["fish"]),text=str(quantity),wanted=wanted))
            x += 100
            if x > 251:
                x = 0
                y += 50

    def TradeLogic(self):
        trader = self.notes["trader"]

        for event in pygame.event.get():
            ## First, give the widgets a chance to react.
            handled = False
            for i in self.interface:
                handled = i.HandleEvent(event)
                if handled:
                    break
            ## If they don't, handle the event here.
            if not handled:
                if event.type == KEYDOWN:
                    if event.key == K_ESCAPE:

                        print "Now leaving the port."
                        print "Trader's values of items:\n"+str(trader.values)
                        print "Trader's estimate of what _you_ value:\n"+str(trader.customer["values"])
                        print "\n\n"

                        trader.StopTrading()

##                        conch.StopMusic()
                        self.PopState()

                    elif event.key == K_c:
                        raise "foo"

                    elif event.key == K_n:
                        trader.comments = []
                        if trader.customer["wants_the_numbers"]:
                            trader.customer["wants_the_numbers"] = False
                        else:
                            trader.customer["wants_the_numbers"] = True
                        trader.JudgeDeal(self.player.offer)
                        self.interface_index["wTalk"].SetText(" ".join(trader.comments))

##                elif event.type == MUSIC_END_EVENT:
##                    conch.PlaySong()

        ## Gameplay events.
        for message in self.messages:
            headline = message.get("headline")
            if headline == "Clicked":
                sender = message.get("sender")

                if sender.name.startswith("item_"):
                    """Add this to the offer, if it fits the offer box
                    (max 6 item types) and if you're not proposing to give
                    the trader more than they can hold."""
                    item = sender.name[5:]
                    multiple = self.notes["goods_multiple"]
                    offer = self.player.offer
                    amount_offered = min(self.player.inventory[item],multiple)
                    total_offered = amount_offered + self.player.offer.get(item,0)
                    add_okay = offer.CanAdd(item,amount_offered) and trader.inventory.CanAdd(item,total_offered)
                    if add_okay:
                        offer.Add(item,amount_offered)
                        self.player.inventory.Remove(item,amount_offered)
                        self.TradeSetup()

                elif sender.name.startswith("offer_"):
                    item = sender.name[6:]
                    multiple = self.notes["goods_multiple"]
                    offer = self.player.offer
                    amount_withdrawn = min(offer[item],multiple)
                    offer.Remove(item,amount_withdrawn)
                    self.player.inventory.Add(item,amount_withdrawn)
                    self.TradeSetup()

                elif sender.name.startswith("trader_offer_"):
                    item = sender.name[13:]
                    print "clicked on trader-offered item: "+item
                    trader.IndicateItemWanted(item,-1)
                    self.TradeSetup()

                elif sender.name.startswith("trader_item_"):
                    item = sender.name[12:]
                    print "clicked on trader-owned item: "+item
                    if item in trader.customer["wants"]:
                        trader.IndicateItemWanted(item,0)
                    elif item in trader.customer["does_not_want"]:
                        trader.IndicateItemWanted(item,0)
                    else:
                        trader.IndicateItemWanted(item,1)
                    self.TradeSetup()

                elif sender.name == "bMultiple":
                    multiple = self.notes["goods_multiple"]
                    if multiple == 1:
                        self.notes["goods_multiple"] = 5
                    elif multiple == 5:
                        self.notes["goods_multiple"] = 10
                    else:
                        self.notes["goods_multiple"] = 1
                    self.interface_index["bMultiple"].SetText("(X"+str(self.notes["goods_multiple"])+")")

                elif sender.name == "bClearOffer":
                    for item in self.player.offer:
                        self.player.inventory[item] += self.player.offer[item]
                        del self.player.offer[item]
                    self.TradeSetup()

                elif sender.name == "bOffer":
                    offer = self.player.offer
                    print "Offering: "+str(offer)
                    print "Appraised at: "+str(trader.Appraise(offer))
                    trader.MakeOffer(offer)
                    print "Trader offers: "+str(trader.offer)
                    ## Make sure the trader isn't offering too much to hold.
                    for item in trader.offer:
                        max_player_can_hold = (self.player.inventory.max_item_quantity or 999999) - self.player.inventory.get(item,0)
                        if trader.offer[item] > max_player_can_hold:
                            n = trader.offer[item] - max_player_can_hold
                            trader.inventory.Add(item,n)
                            trader.offer.Remove(item,n)
                            print "Cleanup: Withdrew "+item+" x"+str(n)+" to prevent player inventory overflow."
                    self.interface_index["wTalk"].SetText(" ".join(trader.comments))
                    self.TradeSetup()

                elif sender.name == "bAccept":
                    trader.JudgeDeal(self.player.offer)
                    trade_ok = self.player.inventory.AddThings(trader.offer)
                    print "Trade OK: "+str(trade_ok)
                    trader.FinalizeDeal(self.player.offer)
                    self.player.offer.Clear()
                    trader.comments = ["Deal! Shall we continue?"]
                    self.TradeSetup()

    def TradeDraw(self):
        screen.fill((0,0,0))
        for i in self.interface:
            i.Draw()


    def MapSetup(self):
        self.AddWidget(driftwood.Widget(name="wHeader",text="Select a port to visit.",coords=(0,550,800,50)))

        for port in PORTS:
            coords = port[1]
            self.AddWidget(driftwood.Button(name="port_"+port[0],text=port[0],coords=(coords[0],coords[1],150,30),border=None))

    def MapLogic(self):
        for event in pygame.event.get():
            ## First, give the widgets a chance to react.
            handled = False
            for i in self.interface:
                handled = i.HandleEvent(event)
                if handled:
                    break
            ## If they don't, handle the event here.
            if not handled:
                if event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
##                        conch.StopMusic()
                        self.PopState()

##                elif event.type == MUSIC_END_EVENT:
##                    conch.PlaySong()

        ## Gameplay events.
        for message in self.messages:
            headline = message.get("headline")
            if headline == "Clicked":
                sender = message.get("sender")
                port_name = sender.name[5:]
                print "Going to port: "+str(port_name)
                self.notes["trader"] = self.world.ports[port_name]["trader"]
                self.notes["trader"].StartTrading(self.player)
                self.PushState("Trade",["trader"])

    def MapDraw(self):
##        screen.fill((0,0,128))
        screen.blit(art.posters["trade_sea"],(0,0))
        for i in self.interface:
            i.Draw()


##___________________________________
##            ~Functions~
##___________________________________


##__________________________________
##            ~Autorun~
##__________________________________

if __name__ == "__main__":

    g = Game()
    g.MainLoop()

    print "\n\n\nGame over. Thanks for playing!"
